@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,8 +13,10 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
+import copy
+
 from robot.api import logger
-from robot.utils import (is_dict_like, is_string, is_truthy, plural_or_not,
+from robot.utils import (is_dict_like, is_list_like, is_number, is_string, is_truthy, plural_or_not,
                          seq2str, seq2str2, type_name, unic, Matcher)
 from robot.utils.asserts import assert_equal
 from robot.version import get_version
@@ -45,6 +48,7 @@ class _List(object):
         | ${L1} = ['a', 'xxx']
         | ${L2} = ['a', 'b', 'x', 'y', 'z']
         """
+        self._validate_list(list_)
         for value in values:
             list_.append(value)
 
@@ -69,6 +73,7 @@ class _List(object):
         | ${L1} = ['xxx', 'a']
         | ${L2} = ['a', 'xxx', 'b']
         """
+        self._validate_list(list_)
         list_.insert(self._index_to_int(index), value)
 
     def combine_lists(self, *lists):
@@ -84,6 +89,7 @@ class _List(object):
         | ${y} = ['a', 'a', 'b', 'a']
         | ${L1} and ${L2} are not changed.
         """
+        self._validate_lists(*lists)
         ret = []
         for item in lists:
             ret.extend(item)
@@ -104,6 +110,7 @@ class _List(object):
         =>
         | ${L3} = ['a', 'xxx', 'yyy']
         """
+        self._validate_list(list_)
         try:
             list_[self._index_to_int(index)] = value
         except IndexError:
@@ -119,6 +126,7 @@ class _List(object):
         =>
         | ${L4} = ['b', 'd']
         """
+        self._validate_list(list_)
         for value in values:
             while value in list_:
                 list_.remove(value)
@@ -138,6 +146,7 @@ class _List(object):
         | ${x} = 'a'
         | ${L2} = ['b']
         """
+        self._validate_list(list_)
         try:
             return list_.pop(self._index_to_int(index))
         except IndexError:
@@ -150,9 +159,8 @@ class _List(object):
         list so that one item can appear only once. Order of the items in
         the new list is the same as in the original except for missing
         duplicates. Number of the removed duplicates is logged.
-
-        New in Robot Framework 2.7.5.
         """
+        self._validate_list(list_)
         ret = []
         for item in list_:
             if item not in ret:
@@ -180,6 +188,7 @@ class _List(object):
         | ${y} = 'd'
         | ${L5} is not changed
         """
+        self._validate_list(list_)
         try:
             return list_[self._index_to_int(index)]
         except IndexError:
@@ -209,6 +218,7 @@ class _List(object):
         | ${z} = ['a', 'b', 'c']
         | ${L5} is not changed
         """
+        self._validate_list(list_)
         start = self._index_to_int(start, True)
         if end is not None:
             end = self._index_to_int(end)
@@ -227,6 +237,7 @@ class _List(object):
         | ${x} = 1
         | ${L3} is not changed
         """
+        self._validate_list(list_)
         return self.get_slice_from_list(list_, start, end).count(value)
 
     def get_index_from_list(self, list_, value, start=0, end=None):
@@ -243,6 +254,7 @@ class _List(object):
         | ${x} = 3
         | ${L5} is not changed
         """
+        self._validate_list(list_)
         if start == '':
             start = 0
         list_ = self.get_slice_from_list(list_, start, end)
@@ -251,11 +263,17 @@ class _List(object):
         except ValueError:
             return -1
 
-    def copy_list(self, list_):
+    def copy_list(self, list_, deepcopy=False):
         """Returns a copy of the given list.
 
+        If the optional ``deepcopy`` is given a true value, the returned
+        list is a deep copy. New option in Robot Framework 3.1.2.
+
         The given list is never altered by this keyword.
         """
+        self._validate_list(list_)
+        if deepcopy:
+            return copy.deepcopy(list_)
         return list_[:]
 
     def reverse_list(self, list_):
@@ -268,38 +286,37 @@ class _List(object):
         =>
         | ${L3} = ['c', 'b', 'a']
         """
+        self._validate_list(list_)
         list_.reverse()
 
     def sort_list(self, list_):
         """Sorts the given list in place.
 
-        The strings are sorted alphabetically and the numbers numerically.
+        Sorting fails if items in the list are not comparable with each others.
+        On Python 2 most objects are comparable, but on Python 3 comparing,
+        for example, strings with numbers is not possible.
 
         Note that the given list is changed and nothing is returned. Use
         `Copy List` first, if you need to keep also the original order.
-
-        ${L} = [2,1,'a','c','b']
-        | Sort List | ${L} |
-        =>
-        | ${L} = [1, 2, 'a', 'b', 'c']
         """
+        self._validate_list(list_)
         list_.sort()
 
     def list_should_contain_value(self, list_, value, msg=None):
         """Fails if the ``value`` is not found from ``list``.
 
-        If the keyword fails, the default error messages is ``<list> does
-        not contain value '<value>'``. A custom message can be given using
-        the ``msg`` argument.
+        Use the ``msg`` argument to override the default error message.
         """
+        self._validate_list(list_)
         default = "%s does not contain value '%s'." % (seq2str2(list_), value)
         _verify_condition(value in list_, default, msg)
 
     def list_should_not_contain_value(self, list_, value, msg=None):
-        """Fails if the ``value`` is not found from ``list``.
+        """Fails if the ``value`` is found from ``list``.
 
-        See `List Should Contain Value` for an explanation of ``msg``.
+        Use the ``msg`` argument to override the default error message.
         """
+        self._validate_list(list_)
         default = "%s contains value '%s'." % (seq2str2(list_), value)
         _verify_condition(value not in list_, default, msg)
 
@@ -314,6 +331,7 @@ class _List(object):
         This keyword works with all iterables that can be converted to a list.
         The original iterable is never altered.
         """
+        self._validate_list(list_)
         if not isinstance(list_, list):
             list_ = list(list_)
         dupes = []
@@ -337,7 +355,6 @@ class _List(object):
         Abc``. The types of the lists do not need to be the same. For example,
         Python tuple and list with same content are considered equal.
 
-
         The error message can be configured using ``msg`` and ``values``
         arguments:
         - If ``msg`` is not given, the default error message is used.
@@ -364,6 +381,7 @@ class _List(object):
         message would contain a row like ``Index 2 (email): name@foo.com !=
         name@bar.com``.
         """
+        self._validate_lists(list1, list2)
         len1 = len(list1)
         len2 = len(list2)
         default = 'Lengths are different: %d != %d' % (len1, len2)
@@ -397,6 +415,7 @@ class _List(object):
         See `Lists Should Be Equal` for more information about configuring
         the error message with ``msg`` and ``values`` arguments.
         """
+        self._validate_lists(list1, list2)
         diffs = ', '.join(unic(item) for item in list2 if item not in list1)
         default = 'Following values were not found from first list: ' + diffs
         _verify_condition(not diffs, default, msg, values)
@@ -409,6 +428,7 @@ class _List(object):
         If you only want to the length, use keyword `Get Length` from
         the BuiltIn library.
         """
+        self._validate_list(list_)
         logger.write('\n'.join(self._log_list(list_)), level)
 
     def _log_list(self, list_):
@@ -433,14 +453,26 @@ class _List(object):
         raise IndexError('Given index %s is out of the range 0-%d.'
                          % (index, len(list_)-1))
 
+    def _validate_list(self, list_, position=1):
+        if not is_list_like(list_):
+            raise TypeError("Expected argument %d to be a list or list-like, "
+                            "got %s instead." % (position, type_name(list_)))
+
+    def _validate_lists(self, *lists):
+        for index, item in enumerate(lists, start=1):
+            self._validate_list(item, index)
+
 
 class _Dictionary(object):
 
     def convert_to_dictionary(self, item):
         """Converts the given ``item`` to a Python ``dict`` type.
 
-        Mainly useful for converting other mappings to dictionaries. Use
-        `Create Dictionary` from the BuiltIn library for constructing new
+        Mainly useful for converting other mappings to normal dictionaries.
+        This includes converting Robot Framework's own ``DotDict`` instances
+        that it uses if variables are created using the ``&{var}`` syntax.
+
+        Use `Create Dictionary` from the BuiltIn library for constructing new
         dictionaries.
 
         New in Robot Framework 2.9.
@@ -457,9 +489,6 @@ class _Dictionary(object):
         =>
         | ${D1} = {'a': 1, 'key': 'value', 'second': 2}
 
-        Starting from Robot Framework 2.8.1, items can also be given as kwargs
-        using ``key=value`` syntax:
-
         | Set To Dictionary | ${D1} | key=value | second=${2} |
 
         The latter syntax is typically more convenient to use, but it has
@@ -467,6 +496,7 @@ class _Dictionary(object):
 
         If given keys already exist in the dictionary, their values are updated.
         """
+        self._validate_dictionary(dictionary)
         if len(key_value_pairs) % 2 != 0:
             raise ValueError("Adding data to a dictionary failed. There "
                              "should be even number of key-value-pairs.")
@@ -486,6 +516,7 @@ class _Dictionary(object):
         =>
         | ${D3} = {'a': 1, 'c': 3}
         """
+        self._validate_dictionary(dictionary)
         for key in keys:
             if key in dictionary:
                 value = dictionary.pop(key)
@@ -508,6 +539,7 @@ class _Dictionary(object):
 
         New in Robot Framework 2.9.2.
         """
+        self._validate_dictionary(dictionary)
         if default is NOT_SET:
             self.dictionary_should_contain_key(dictionary, key)
             return dictionary.pop(key)
@@ -524,62 +556,110 @@ class _Dictionary(object):
         =>
         | ${D5} = {'b': 2, 'd': 4}
         """
+        self._validate_dictionary(dictionary)
         remove_keys = [k for k in dictionary if k not in keys]
         self.remove_from_dictionary(dictionary, *remove_keys)
 
-    def copy_dictionary(self, dictionary):
+    def copy_dictionary(self, dictionary, deepcopy=False):
         """Returns a copy of the given dictionary.
 
+        The ``deepcopy`` argument controls should the returned dictionary be
+        a [https://docs.python.org/library/copy.html|shallow or deep copy].
+        By default returns a shallow copy, but that can be changed by giving
+        ``deepcopy`` a true value (see `Boolean arguments`). This is a new
+        option in Robot Framework 3.1.2. Earlier versions always returned
+        shallow copies.
+
         The given dictionary is never altered by this keyword.
         """
+        self._validate_dictionary(dictionary)
+        if deepcopy:
+            return copy.deepcopy(dictionary)
         return dictionary.copy()
 
-    def get_dictionary_keys(self, dictionary):
-        """Returns keys of the given ``dictionary``.
+    def get_dictionary_keys(self, dictionary, sort_keys=True):
+        """Returns keys of the given ``dictionary`` as a list.
 
-        If keys are sortable, they are returned in sorted order. The given
-        ``dictionary`` is never altered by this keyword.
+        By default keys are returned in sorted order (assuming they are
+        sortable), but they can be returned in the original order by giving
+        ``sort_keys``  a false value (see `Boolean arguments`). Notice that
+        with Python 3.5 and earlier dictionary order is undefined unless using
+        ordered dictionaries.
+
+        The given ``dictionary`` is never altered by this keyword.
 
         Example:
-        | ${keys} = | Get Dictionary Keys | ${D3} |
+        | ${sorted} =   | Get Dictionary Keys | ${D3} |
+        | ${unsorted} = | Get Dictionary Keys | ${D3} | sort_keys=False |
         =>
-        | ${keys} = ['a', 'b', 'c']
+        | ${sorted} = ['a', 'b', 'c']
+        | ${unsorted} = ['b', 'a', 'c']   # Order depends on Python version.
+
+        ``sort_keys`` is a new option in Robot Framework 3.1.2. Earlier keys
+        were always sorted.
         """
-        # TODO: Possibility to disable sorting. Can be handy with OrderedDicts.
+        self._validate_dictionary(dictionary)
         keys = dictionary.keys()
-        try:
-            return sorted(keys)
-        except TypeError:
-            return list(keys)
+        if sort_keys:
+            try:
+                return sorted(keys)
+            except TypeError:
+                pass
+        return list(keys)
 
-    def get_dictionary_values(self, dictionary):
-        """Returns values of the given dictionary.
+    def get_dictionary_values(self, dictionary, sort_keys=True):
+        """Returns values of the given ``dictionary`` as a list.
 
-        Values are returned sorted according to keys. The given dictionary is
-        never altered by this keyword.
+        Uses `Get Dictionary Keys` to get keys and then returns corresponding
+        values. By default keys are sorted and values returned in that order,
+        but this can be changed by giving ``sort_keys`` a false value (see
+        `Boolean arguments`). Notice that with Python 3.5 and earlier
+        dictionary order is undefined unless using ordered dictionaries.
+
+        The given ``dictionary`` is never altered by this keyword.
 
         Example:
-        | ${values} = | Get Dictionary Values | ${D3} |
+        | ${sorted} =   | Get Dictionary Values | ${D3} |
+        | ${unsorted} = | Get Dictionary Values | ${D3} | sort_keys=False |
         =>
-        | ${values} = [1, 2, 3]
+        | ${sorted} = [1, 2, 3]
+        | ${unsorted} = [2, 1, 3]    # Order depends on Python version.
+
+        ``sort_keys`` is a new option in Robot Framework 3.1.2. Earlier values
+        were always sorted based on keys.
         """
-        return [dictionary[k] for k in self.get_dictionary_keys(dictionary)]
+        self._validate_dictionary(dictionary)
+        keys = self.get_dictionary_keys(dictionary, sort_keys=sort_keys)
+        return [dictionary[k] for k in keys]
 
-    def get_dictionary_items(self, dictionary):
-        """Returns items of the given ``dictionary``.
+    def get_dictionary_items(self, dictionary, sort_keys=True):
+        """Returns items of the given ``dictionary`` as a list.
 
-        Items are returned sorted by keys. The given ``dictionary`` is not
-        altered by this keyword.
+        Uses `Get Dictionary Keys` to get keys and then returns corresponding
+        items. By default keys are sorted and items returned in that order,
+        but this can be changed by giving ``sort_keys`` a false value (see
+        `Boolean arguments`). Notice that with Python 3.5 and earlier
+        dictionary order is undefined unless using ordered dictionaries.
+
+        Items are returned as a flat list so that first item is a key,
+        second item is a corresponding value, third item is the second key,
+        and so on.
+
+        The given ``dictionary`` is never altered by this keyword.
 
         Example:
-        | ${items} = | Get Dictionary Items | ${D3} |
+        | ${sorted} =   | Get Dictionary Items | ${D3} |
+        | ${unsorted} = | Get Dictionary Items | ${D3} | sort_keys=False |
         =>
-        | ${items} = ['a', 1, 'b', 2, 'c', 3]
+        | ${sorted} = ['a', 1, 'b', 2, 'c', 3]
+        | ${unsorted} = ['b', 2, 'a', 1, 'c', 3]    # Order depends on Python version.
+
+        ``sort_keys`` is a new option in Robot Framework 3.1.2. Earlier items
+        were always sorted based on keys.
         """
-        ret = []
-        for key in self.get_dictionary_keys(dictionary):
-            ret.extend((key, dictionary[key]))
-        return ret
+        self._validate_dictionary(dictionary)
+        keys = self.get_dictionary_keys(dictionary, sort_keys=sort_keys)
+        return [i for key in keys for i in (key, dictionary[key])]
 
     def get_from_dictionary(self, dictionary, key):
         """Returns a value from the given ``dictionary`` based on the given ``key``.
@@ -594,6 +674,7 @@ class _Dictionary(object):
         =>
         | ${value} = 2
         """
+        self._validate_dictionary(dictionary)
         try:
             return dictionary[key]
         except KeyError:
@@ -602,31 +683,29 @@ class _Dictionary(object):
     def dictionary_should_contain_key(self, dictionary, key, msg=None):
         """Fails if ``key`` is not found from ``dictionary``.
 
-        See `List Should Contain Value` for an explanation of ``msg``.
-
-        The given dictionary is never altered by this keyword.
+        Use the ``msg`` argument to override the default error message.
         """
+        self._validate_dictionary(dictionary)
         default = "Dictionary does not contain key '%s'." % key
         _verify_condition(key in dictionary, default, msg)
 
     def dictionary_should_not_contain_key(self, dictionary, key, msg=None):
         """Fails if ``key`` is found from ``dictionary``.
 
-        See `List Should Contain Value` for an explanation of ``msg``.
-
-        The given dictionary is never altered by this keyword.
+        Use the ``msg`` argument to override the default error message.
         """
+        self._validate_dictionary(dictionary)
         default = "Dictionary contains key '%s'." % key
         _verify_condition(key not in dictionary, default, msg)
 
     def dictionary_should_contain_item(self, dictionary, key, value, msg=None):
-        """An item of ``key``/``value`` must be found in a `dictionary`.
+        """An item of ``key`` / ``value`` must be found in a ``dictionary``.
 
         Value is converted to unicode for comparison.
 
-        See `Lists Should Be Equal` for an explanation of ``msg``.
-        The given dictionary is never altered by this keyword.
+        Use the ``msg`` argument to override the default error message.
         """
+        self._validate_dictionary(dictionary)
         self.dictionary_should_contain_key(dictionary, key, msg)
         actual, expected = unic(dictionary[key]), unic(value)
         default = "Value of dictionary key '%s' does not match: %s != %s" % (key, actual, expected)
@@ -635,20 +714,18 @@ class _Dictionary(object):
     def dictionary_should_contain_value(self, dictionary, value, msg=None):
         """Fails if ``value`` is not found from ``dictionary``.
 
-        See `List Should Contain Value` for an explanation of ``msg``.
-
-        The given dictionary is never altered by this keyword.
+        Use the ``msg`` argument to override the default error message.
         """
+        self._validate_dictionary(dictionary)
         default = "Dictionary does not contain value '%s'." % value
         _verify_condition(value in dictionary.values(), default, msg)
 
     def dictionary_should_not_contain_value(self, dictionary, value, msg=None):
         """Fails if ``value`` is found from ``dictionary``.
 
-        See `List Should Contain Value` for an explanation of ``msg``.
-
-        The given dictionary is never altered by this keyword.
+        Use the ``msg`` argument to override the default error message.
         """
+        self._validate_dictionary(dictionary)
         default = "Dictionary contains value '%s'." % value
         _verify_condition(not value in dictionary.values(), default, msg)
 
@@ -662,9 +739,9 @@ class _Dictionary(object):
 
         See `Lists Should Be Equal` for more information about configuring
         the error message with ``msg`` and ``values`` arguments.
-
-        The given dictionaries are never altered by this keyword.
         """
+        self._validate_dictionary(dict1)
+        self._validate_dictionary(dict2, 2)
         keys = self._keys_should_be_equal(dict1, dict2, msg, values)
         self._key_values_should_be_equal(keys, dict1, dict2, msg, values)
 
@@ -674,9 +751,9 @@ class _Dictionary(object):
 
         See `Lists Should Be Equal` for more information about configuring
         the error message with ``msg`` and ``values`` arguments.
-
-        The given dictionaries are never altered by this keyword.
         """
+        self._validate_dictionary(dict1)
+        self._validate_dictionary(dict2, 2)
         keys = self.get_dictionary_keys(dict2)
         diffs = [unic(k) for k in keys if k not in dict1]
         default = "Following keys missing from first dictionary: %s" \
@@ -692,6 +769,7 @@ class _Dictionary(object):
         If you only want to log the size, use keyword `Get Length` from
         the BuiltIn library.
         """
+        self._validate_dictionary(dictionary)
         logger.write('\n'.join(self._log_dictionary(dictionary)), level)
 
     def _log_dictionary(self, dictionary):
@@ -731,6 +809,11 @@ class _Dictionary(object):
             except AssertionError as err:
                 yield unic(err)
 
+    def _validate_dictionary(self, dictionary, position=1):
+        if is_string(dictionary) or is_number(dictionary):
+            raise TypeError("Expected argument %d to be a dictionary or dictionary-like, "
+                            "got %s instead." % (position, type_name(dictionary)))
+
 
 class Collections(_List, _Dictionary):
     """A test library providing keywords for handling lists and dictionaries.
@@ -775,13 +858,12 @@ class Collections(_List, _Dictionary):
 
     Some keywords accept arguments that are handled as Boolean values true or
     false. If such an argument is given as a string, it is considered false if
-    it is either empty or case-insensitively equal to ``false`` or ``no``.
-    Keywords verifying something that allow dropping actual and expected values
-    from the possible error message also consider string ``no values`` as false.
-    Other strings are considered true regardless their value, and other
-    argument types are tested using same
-    [http://docs.python.org/2/library/stdtypes.html#truth-value-testing|rules
-    as in Python].
+    it is an empty string or equal to ``FALSE``, ``NONE``, ``NO``, ``OFF`` or
+    ``0``, case-insensitively. Keywords verifying something that allow dropping
+    actual and expected values from the possible error message also consider
+    string ``no values`` to be false. Other strings are considered true
+    regardless their value, and other argument types are tested using the same
+    [http://docs.python.org/library/stdtypes.html#truth|rules as in Python].
 
     True examples:
     | `Should Contain Match` | ${list} | ${pattern} | case_insensitive=True    | # Strings are generally true.    |
@@ -796,8 +878,8 @@ class Collections(_List, _Dictionary):
     | `Should Contain Match` | ${list} | ${pattern} | case_insensitive=${FALSE} | # Python ``False`` is false.   |
     | `Lists Should Be Equal` | ${x}   | ${y} | Custom error | values=no values | # ``no values`` works with ``values`` argument |
 
-    Note that prior to Robot Framework 2.9 some keywords considered all
-    non-empty strings, including ``False``, to be true.
+    Considering string ``NONE`` false is new in Robot Framework 3.0.3 and
+    considering also ``OFF`` and ``0`` false is new in Robot Framework 3.1.
 
     = Data in examples =
 
@@ -817,8 +899,6 @@ class Collections(_List, _Dictionary):
                              whitespace_insensitive=False):
         """Fails if ``pattern`` is not found in ``list``.
 
-        See `List Should Contain Value` for an explanation of ``msg``.
-
         By default, pattern matching is similar to matching files in a shell
         and is case-sensitive and whitespace-sensitive. In the pattern syntax,
         ``*`` matches to anything and ``?`` matches to any single character. You
@@ -827,7 +907,7 @@ class Collections(_List, _Dictionary):
 
         If you prepend ``regexp=`` to your pattern, your pattern will be used
         according to the Python
-        [http://docs.python.org/2/library/re.html|re module] regular expression
+        [http://docs.python.org/library/re.html|re module] regular expression
         syntax. Important note: Backslashes are an escape character, and must
         be escaped with another backslash (e.g. ``regexp=\\\\d{6}`` to search for
         ``\\d{6}``). See `BuiltIn.Should Match Regexp` for more details.
@@ -840,7 +920,7 @@ class Collections(_List, _Dictionary):
 
         Non-string values in lists are ignored when matching patterns.
 
-        The given list is never altered by this keyword.
+        Use the ``msg`` argument to override the default error message.
 
         See also ``Should Not Contain Match``.
 
@@ -851,9 +931,8 @@ class Collections(_List, _Dictionary):
         | Should Contain Match | ${list} | a*  | case_insensitive=True       | | # Match strings beginning with 'a' or 'A'. |
         | Should Contain Match | ${list} | ab* | whitespace_insensitive=yes  | | # Match strings beginning with 'ab' with possible whitespace ignored. |
         | Should Contain Match | ${list} | ab* | whitespace_insensitive=true | case_insensitive=true | # Same as the above but also ignore case. |
-
-        New in Robot Framework 2.8.6.
         """
+        _List._validate_list(self, list)
         matches = _get_matches_in_iterable(list, pattern, case_insensitive,
                                            whitespace_insensitive)
         default = "%s does not contain match for pattern '%s'." \
@@ -867,9 +946,8 @@ class Collections(_List, _Dictionary):
 
         Exact opposite of `Should Contain Match` keyword. See that keyword
         for information about arguments and usage in general.
-
-        New in Robot Framework 2.8.6.
         """
+        _List._validate_list(self, list)
         matches = _get_matches_in_iterable(list, pattern, case_insensitive,
                                            whitespace_insensitive)
         default = "%s contains match for pattern '%s'." \
@@ -887,9 +965,8 @@ class Collections(_List, _Dictionary):
         | ${matches}= | Get Matches | ${list} | a* | # ${matches} will contain any string beginning with 'a' |
         | ${matches}= | Get Matches | ${list} | regexp=a.* | # ${matches} will contain any string beginning with 'a' (regexp version) |
         | ${matches}= | Get Matches | ${list} | a* | case_insensitive=${True} | # ${matches} will contain any string beginning with 'a' or 'A' |
-
-        New in Robot Framework 2.8.6.
         """
+        _List._validate_list(self, list)
         return _get_matches_in_iterable(list, pattern, case_insensitive,
                                         whitespace_insensitive)
 
@@ -904,9 +981,8 @@ class Collections(_List, _Dictionary):
         | ${count}= | Get Match Count | ${list} | a* | # ${count} will be the count of strings beginning with 'a' |
         | ${count}= | Get Match Count | ${list} | regexp=a.* | # ${matches} will be the count of strings beginning with 'a' (regexp version) |
         | ${count}= | Get Match Count | ${list} | a* | case_insensitive=${True} | # ${matches} will be the count of strings beginning with 'a' or 'A' |
-
-        New in Robot Framework 2.8.6.
         """
+        _List._validate_list(self, list)
         return len(self.get_matches(list, pattern, case_insensitive,
                                     whitespace_insensitive))
 
