@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -28,7 +29,7 @@ from robot.utils import (abspath, ConnectionCache, console_decode, del_env_var,
                          is_unicode, normpath, parse_time, plural_or_not,
                          secs_to_timestamp, secs_to_timestr, seq2str,
                          set_env_var, timestr_to_secs, unic, CONSOLE_ENCODING,
-                         IRONPYTHON, PY2, SYSTEM_ENCODING)
+                         IRONPYTHON, JYTHON, PY2, PY3, SYSTEM_ENCODING, WINDOWS)
 
 __version__ = get_version()
 PROCESSES = ConnectionCache('No active processes.')
@@ -73,17 +74,18 @@ class OperatingSystem(object):
 
     = Pattern matching =
 
-    Some keywords allow their arguments to be specified as _glob patterns_
-    where:
-    | ``*``        | matches anything, even an empty string |
-    | ``?``        | matches any single character |
-    | ``[chars]``  | matches any character inside square brackets (e.g. ``[abc]`` matches either ``a``, ``b`` or ``c``) |
-    | ``[!chars]`` | matches any character not inside square brackets |
+    Some keywords allow their arguments to be specified as
+    [http://en.wikipedia.org/wiki/Glob_(programming)|glob patterns] where:
+
+    | ``*``        | matches any string, even an empty string                |
+    | ``?``        | matches any single character                            |
+    | ``[chars]``  | matches one character in the bracket                    |
+    | ``[!chars]`` | matches one character not in the bracket                |
+    | ``[a-z]``    | matches one character from the range in the bracket     |
+    | ``[!a-z]``   | matches one character not from the range in the bracket |
 
     Unless otherwise noted, matching is case-insensitive on
-    case-insensitive operating systems such as Windows. Pattern
-    matching is implemented using
-    [http://docs.python.org/library/fnmatch.html|fnmatch module].
+    case-insensitive operating systems such as Windows.
 
     Starting from Robot Framework 2.9.1, globbing is not done if the given path
     matches an existing file even if it would contain a glob pattern.
@@ -96,18 +98,16 @@ class OperatingSystem(object):
     ``C:\\Users\\<user>\\robot`` on Windows and ``/home/<user>/robot`` on
     Unixes.
 
-    Tilde expansion is a new feature in Robot Framework 2.8. The ``~username``
-    form does not work on Jython
+    The ``~username`` form does not work on Jython.
 
     = Boolean arguments =
 
     Some keywords accept arguments that are handled as Boolean values true or
     false. If such an argument is given as a string, it is considered false if
-    it is either empty or case-insensitively equal to ``false`` or ``no``.
-    Other strings are considered true regardless their value, and other
-    argument types are tested using same
-    [http://docs.python.org/2/library/stdtypes.html#truth-value-testing|rules
-    as in Python].
+    it is an empty string or equal to ``FALSE``, ``NONE``, ``NO``, ``OFF`` or
+    ``0``, case-insensitively. Other strings are considered true regardless
+    their value, and other argument types are tested using the same
+    [http://docs.python.org/library/stdtypes.html#truth|rules as in Python].
 
     True examples:
     | `Remove Directory` | ${path} | recursive=True    | # Strings are generally true.    |
@@ -121,8 +121,8 @@ class OperatingSystem(object):
     | `Remove Directory` | ${path} | recursive=${EMPTY} | # Empty string is false.       |
     | `Remove Directory` | ${path} | recursive=${FALSE} | # Python ``False`` is false.   |
 
-    Note that prior to Robot Framework 2.9, all non-empty strings, including
-    ``false`` and ``no``, were considered true.
+    Considering string ``NONE`` false is new in Robot Framework 3.0.3 and
+    considering also ``OFF`` and ``0`` false is new in Robot Framework 3.1.
 
     = Example =
 
@@ -261,8 +261,7 @@ class OperatingSystem(object):
         - ``replace``: Replace characters that cannot be decoded with
           a replacement character.
 
-        ``encoding_errors`` argument was added in Robot Framework 2.8.5 and the
-        support for ``SYSTEM`` and ``CONSOLE`` encodings in Robot Framework 3.0.
+        Support for ``SYSTEM`` and ``CONSOLE`` encodings in Robot Framework 3.0.
         """
         path = self._absnorm(path)
         self._link("Getting file '%s'.", path)
@@ -278,6 +277,9 @@ class OperatingSystem(object):
         return content.replace('\r\n', '\n')
 
     def _map_encoding(self, encoding):
+        # Python 3 opens files in native system encoding by default.
+        if PY3 and encoding.upper() == 'SYSTEM':
+            return None
         return {'SYSTEM': SYSTEM_ENCODING,
                 'CONSOLE': CONSOLE_ENCODING}.get(encoding.upper(), encoding)
 
@@ -314,8 +316,6 @@ class OperatingSystem(object):
         If more complex pattern matching is needed, it is possible to use
         `Get File` in combination with String library keywords like `Get
         Lines Matching Regexp`.
-
-        ``encoding_errors`` argument is new in Robot Framework 2.8.5.
         """
         pattern = '*%s*' % pattern
         path = self._absnorm(path)
@@ -340,8 +340,6 @@ class OperatingSystem(object):
 
         See `Get File` for more information about ``encoding`` and
         ``encoding_errors`` arguments.
-
-        ``encoding_errors`` argument is new in Robot Framework 2.8.5.
         """
         content = self.get_file(path, encoding, encoding_errors)
         self._info(content)
@@ -549,16 +547,20 @@ class OperatingSystem(object):
     def create_file(self, path, content='', encoding='UTF-8'):
         """Creates a file with the given content and encoding.
 
-        If the directory where to create file does not exist it, and possible
-        intermediate missing directories, are created.
+        If the directory where the file is created does not exist, it is
+        automatically created along with possible missing intermediate
+        directories. Possible existing file is overwritten.
+
+        On Windows newline characters (``\\n``) in content are automatically
+        converted to Windows native newline sequence (``\\r\\n``).
 
         See `Get File` for more information about possible ``encoding`` values,
         including special values ``SYSTEM`` and ``CONSOLE``.
 
         Examples:
-        | Create File | ${dir}/example.txt | Hello, world!      |         |
-        | Create File | ${path}            | Hyv\\xe4 esimerkki | Latin-1 |
-        | Create File | /tmp/foo.txt       | ${content}         | SYSTEM  |
+        | Create File | ${dir}/example.txt | Hello, world!       |         |
+        | Create File | ${path}            | Hyv\\xe4 esimerkki  | Latin-1 |
+        | Create File | /tmp/foo.txt       | 3\\nlines\\nhere\\n | SYSTEM  |
 
         Use `Append To File` if you want to append to an existing file
         and `Create Binary File` if you need to write bytes without encoding.
@@ -566,9 +568,10 @@ class OperatingSystem(object):
         files.
 
         The support for ``SYSTEM`` and ``CONSOLE`` encodings is new in Robot
-        Framework 3.0.
+        Framework 3.0. Automatically converting ``\\n`` to ``\\r\\n`` on
+        Windows is new in Robot Framework 3.1.
         """
-        path = self._write_to_file(path, content, self._map_encoding(encoding))
+        path = self._write_to_file(path, content, encoding)
         self._link("Created file '%s'.", path)
 
     def _write_to_file(self, path, content, encoding=None, mode='w'):
@@ -576,9 +579,13 @@ class OperatingSystem(object):
         parent = os.path.dirname(path)
         if not os.path.exists(parent):
             os.makedirs(parent)
+        # io.open() only accepts Unicode, not byte-strings, in text mode.
+        # We expect possible byte-strings to be all ASCII.
+        if PY2 and isinstance(content, str) and 'b' not in mode:
+            content = unicode(content)
         if encoding:
-            content = content.encode(encoding)
-        with open(path, mode+'b') as f:
+            encoding = self._map_encoding(encoding)
+        with io.open(path, mode, encoding=encoding) as f:
             f.write(content)
         return path
 
@@ -592,8 +599,8 @@ class OperatingSystem(object):
 
         Byte strings, and possible other types, are written to the file as is.
 
-        If the directory where to create file does not exist it, and possible
-        intermediate missing directories, are created.
+        If the directory for the file does not exist, it is created, along
+        with missing intermediate directories.
 
         Examples:
         | Create Binary File | ${dir}/example.png | ${image content}     |
@@ -602,19 +609,24 @@ class OperatingSystem(object):
         Use `Create File` if you want to create a text file using a certain
         encoding. `File Should Not Exist` can be used to avoid overwriting
         existing files.
-
-        New in Robot Framework 2.8.5.
         """
         if is_unicode(content):
             content = bytes(bytearray(ord(c) for c in content))
-        path = self._write_to_file(path, content)
+        path = self._write_to_file(path, content, mode='wb')
         self._link("Created binary file '%s'.", path)
 
     def append_to_file(self, path, content, encoding='UTF-8'):
-        """Appends the given contend to the specified file.
+        """Appends the given content to the specified file.
 
-        If the file does not exists, this keyword works exactly the same
-        way as `Create File`.
+        If the file exists, the given text is written to its end. If the file
+        does not exist, it is created.
+
+        Other than not overwriting possible existing files, this keyword works
+        exactly like `Create File`. See its documentation for more details
+        about the usage.
+
+        Note that special encodings ``SYSTEM`` and ``CONSOLE`` only work
+        with this keyword starting from Robot Framework 3.1.2.
         """
         path = self._write_to_file(path, content, encoding, mode='a')
         self._link("Appended to file '%s'.", path)
@@ -708,9 +720,9 @@ class OperatingSystem(object):
     def copy_file(self, source, destination):
         """Copies the source file into the destination.
 
-        Source must be an existing file. Starting from Robot Framework 2.8.4,
-        it can be given as a glob pattern (see `Pattern matching`) that matches
-        exactly one file. How the destination is interpreted is explained below.
+        Source must be a path to an existing file or a glob pattern (see
+        `Pattern matching`) that matches exactly one file. How the
+        destination is interpreted is explained below.
 
         1) If the destination is an existing file, the source file is copied
         over it.
@@ -780,7 +792,7 @@ class OperatingSystem(object):
         return False
 
     def _force_normalize(self, path):
-        # TODO: Should normalize_path also support case and link normalization?
+        # TODO: Should normalize_path also support link normalization?
         # TODO: Should we handle dos paths like 'exampl~1.txt'?
         return os.path.realpath(normpath(path, case_normalize=True))
 
@@ -844,8 +856,6 @@ class OperatingSystem(object):
         | Copy Files | ${dir}/file-*.txt | ${dir2}          |         |
 
         See also `Copy File`, `Move File`, and `Move Files`.
-
-        New in Robot Framework 2.8.4.
         """
         sources, destination \
             = self._prepare_copy_and_move_files(sources_and_destination)
@@ -872,8 +882,6 @@ class OperatingSystem(object):
         Arguments have exactly same semantics as with `Copy Files` keyword.
 
         See also `Move File`, `Copy File`, and `Copy Files`.
-
-        New in Robot Framework 2.8.4.
         """
         sources, destination \
             = self._prepare_copy_and_move_files(sources_and_destination)
@@ -889,7 +897,12 @@ class OperatingSystem(object):
         """
         source, destination \
             = self._prepare_copy_and_move_directory(source, destination)
-        shutil.copytree(source, destination)
+        try:
+            shutil.copytree(source, destination)
+        except shutil.Error:
+            # https://github.com/robotframework/robotframework/issues/2321
+            if not (WINDOWS and JYTHON):
+                raise
         self._link("Copied directory from '%s' to '%s'.", source, destination)
 
     def _prepare_copy_and_move_directory(self, source, destination):
@@ -930,8 +943,8 @@ class OperatingSystem(object):
         If no such environment variable is set, returns the default value, if
         given. Otherwise fails the test case.
 
-        Starting from Robot Framework 2.7, returned variables are automatically
-        decoded to Unicode using the system encoding.
+        Returned variables are automatically decoded to Unicode using
+        the system encoding.
 
         Note that you can also access environment variables directly using
         the variable syntax ``%{ENV_VAR_NAME}``.
@@ -944,9 +957,8 @@ class OperatingSystem(object):
     def set_environment_variable(self, name, value):
         """Sets an environment variable to a specified value.
 
-        Values are converted to strings automatically. Starting from Robot
-        Framework 2.7, set variables are automatically encoded using the system
-        encoding.
+        Values are converted to strings automatically. Set variables are
+        automatically encoded using the system encoding.
         """
         set_env_var(name, value)
         self._info("Environment variable '%s' set to value '%s'."
@@ -972,8 +984,6 @@ class OperatingSystem(object):
         | Should Be Equal                | %{NAME2} | first  |                 |
         | Append To Environment Variable | NAME2    | second | separator=-     |
         | Should Be Equal                | %{NAME2} | first-second             |
-
-        New in Robot Framework 2.8.4.
         """
         sentinel = object()
         initial = self.get_environment_variable(name, sentinel)
@@ -991,8 +1001,8 @@ class OperatingSystem(object):
 
         Does nothing if the environment variable is not set.
 
-        Starting from Robot Framework 2.7, it is possible to remove multiple
-        variables by passing them to this keyword as separate arguments.
+        It is possible to remove multiple variables by passing them to this
+        keyword as separate arguments.
         """
         for name in names:
             value = del_env_var(name)
@@ -1028,8 +1038,6 @@ class OperatingSystem(object):
         Both keys and values are decoded to Unicode using the system encoding.
         Altering the returned dictionary has no effect on the actual environment
         variables.
-
-        New in Robot Framework 2.7.
         """
         return get_env_vars()
 
@@ -1038,8 +1046,6 @@ class OperatingSystem(object):
 
         Environment variables are also returned the same way as with
         `Get Environment Variables` keyword.
-
-        New in Robot Framework 2.7.
         """
         variables = get_env_vars()
         for name in sorted(variables, key=lambda item: item.lower()):
@@ -1078,9 +1084,9 @@ class OperatingSystem(object):
         See `Join Path` for more information.
 
         Examples:
-        | @{p1} = | Join Path | base     | example       | other |          |
-        | @{p2} = | Join Path | /my/base | /example      | other |          |
-        | @{p3} = | Join Path | my/base  | example/path/ | other | one/more |
+        | @{p1} = | Join Paths | base     | example       | other |          |
+        | @{p2} = | Join Paths | /my/base | /example      | other |          |
+        | @{p3} = | Join Paths | my/base  | example/path/ | other | one/more |
         =>
         - @{p1} = ['base/example', 'base/other']
         - @{p2} = ['/example', '/my/base/other']
@@ -1088,23 +1094,38 @@ class OperatingSystem(object):
         """
         return [self.join_path(base, path) for path in paths]
 
-    def normalize_path(self, path):
+    def normalize_path(self, path, case_normalize=False):
         """Normalizes the given path.
 
+        - Collapses redundant separators and up-level references.
+        - Converts ``/`` to ``\\`` on Windows.
+        - Replaces initial ``~`` or ``~user`` by that user's home directory.
+          The latter is not supported on Jython.
+        - If ``case_normalize`` is given a true value (see `Boolean arguments`)
+          on Windows, converts the path to all lowercase. New in Robot
+          Framework 3.1.
+
         Examples:
-        | ${path} = | Normalize Path | abc        |
-        | ${p2} =   | Normalize Path | abc/       |
-        | ${p3} =   | Normalize Path | abc/../def |
-        | ${p4} =   | Normalize Path | abc/./def  |
-        | ${p5} =   | Normalize Path | abc//def   |
+        | ${path1} = | Normalize Path | abc/           |
+        | ${path2} = | Normalize Path | abc/../def     |
+        | ${path3} = | Normalize Path | abc/./def//ghi |
+        | ${path4} = | Normalize Path | ~robot/stuff   |
         =>
-        - ${path} = 'abc'
-        - ${p2} = 'abc'
-        - ${p3} = 'def'
-        - ${p4} = 'abc/def'
-        - ${p5} = 'abc/def'
+        - ${path1} = 'abc'
+        - ${path2} = 'def'
+        - ${path3} = 'abc/def/ghi'
+        - ${path4} = '/home/robot/stuff'
+
+        On Windows result would use ``\\`` instead of ``/`` and home directory
+        would be different.
         """
         path = os.path.normpath(os.path.expanduser(path.replace('/', os.sep)))
+        # os.path.normcase doesn't normalize on OSX which also, by default,
+        # has case-insensitive file system. Our robot.utils.normpath would
+        # do that, but it's not certain would that, or other things that the
+        # utility do, desirable.
+        if case_normalize:
+            path = os.path.normcase(path)
         return path or '.'
 
     def split_path(self, path):
@@ -1234,12 +1255,10 @@ class OperatingSystem(object):
            ``YYYYMMDD hhmmss``.
 
         3) If ``mtime`` is equal to ``NOW``, the current local time is used.
-           This time is got using Python's ``time.time()`` function.
 
         4) If ``mtime`` is equal to ``UTC``, the current time in
            [http://en.wikipedia.org/wiki/Coordinated_Universal_Time|UTC]
-           is used. This time is got using ``time.time() + time.altzone``
-           in Python.
+           is used.
 
         5) If ``mtime`` is in the format like ``NOW - 1 day`` or ``UTC + 1
            hour 30 min``, the current local/UTC time plus/minus the time
@@ -1252,8 +1271,6 @@ class OperatingSystem(object):
         | Set Modified Time | /path/file | NOW                | # The local time of execution |
         | Set Modified Time | /path/file | NOW - 1 day        | # 1 day subtracted from the local time |
         | Set Modified Time | /path/file | UTC + 1h 2min 3s   | # 1h 2min 3s added to the UTC time |
-
-        Support for UTC time is a new feature in Robot Framework 2.7.5.
         """
         mtime = parse_time(mtime)
         path = self._absnorm(path)
@@ -1438,7 +1455,7 @@ class _Process:
         #   In Jython return code can be between '-255' - '255'
         #   In Python return code must be converted with 'rc >> 8' and it is
         #   between 0-255 after conversion
-        if os.sep == '\\' or sys.platform.startswith('java'):
+        if WINDOWS or JYTHON:
             return rc % 256
         return rc >> 8
 
